/**==============================================================================
 * 
 *  ====== - Версии - ======
 * 
 * 0.1 - Реализация.
 * 0.2 - Добавлено игнорирование блока слов для админов.
 * 0.3 - Фиксы.
 * 0.4 - Исправлены ошибки в логах
 * 0.5 - Фиксы.
 * 
 *  ====== - Описание - ======
 * 
 * Плагин добавит на ваш сервер возможность игнорировать слова в чате.
 * Если вы хотите запретить слово в чат, впишите его в ignore_words.ini
 * Кол-во вписанных слов не должно превышать значение MAX_WORDS
 * 
 *  ====== - Настройки - ======
 * 
 *  Игнорировать блок слов для админа [ По умолчанию: 0 ]
 *  Да - 1
 *  Нет - 0
 * iw_admin_ignored "0"
 *
 *  Удалять игнорируемое слово или заменить [ По умолчанию: 1 ]
 *  Заменять - 1
 *  Удалять - 0
 * iw_delete_or_replace "1"
 * 
 * ====== - Заменяемое слово - ======
 * 
 * В файле replace_message.ini вписываем слово, которое вы хотите.
 * [ По умолчанию: "Сообщение от ( NAME ) удалено" ]
 *
================================================================================*/

/**==============================================================================
	Инклуды
================================================================================*/
#include < amxmodx >

/**==============================================================================
	Данные
================================================================================*/
#define PLUGIN_AUTHOR 		"BlackSmoke"
#define PLUGIN_VERSION 		"0.5 beta"
#define PLUGIN_NAME 		"Ignore Words"

#define ADMIN_FLAG ADMIN_BAN

#pragma semicolon 1

#define MAX_WORDS 1024

new
s_File[ 64 ],
s_FileReplace[ 64 ],
s_WordString[ MAX_WORDS ][ 64 ],
s_StringReplace[ 256 ] = { "Сообщение от ( NAME ) удалено" },

iPosStrFile,

iCvarIgnoreAdmins,
iCvarDeleteOrReplace;

/**==============================================================================
	Инициализация плагина
================================================================================*/
public plugin_init( )
{
	register_plugin( PLUGIN_NAME, PLUGIN_VERSION, PLUGIN_AUTHOR ) ;
	
	register_clcmd( "say ", "Hook__Say" ) ;
	register_clcmd( "say_team ", "Hook__Say" ) ;
	
	iCvarIgnoreAdmins = register_cvar( "iw_admin_ignored", "0" ) ;
	iCvarDeleteOrReplace = register_cvar( "iw_delete_or_replace", "1" ) ;
}

/**==============================================================================
	Загрузки плагина
================================================================================*/
public plugin_precache( )
{
	format( s_File, charsmax( s_File ), "addons/amxmodx/configs/ignore_words/ignore_words.ini" );
	format( s_FileReplace, charsmax( s_FileReplace ), "addons/amxmodx/configs/ignore_words/replace_message.ini" );
	new i, iLen;
	if( !dir_exists( "addons/amxmodx/configs/ignore_words" ) )
	{
		return PLUGIN_CONTINUE;
	}
	
	if( file_exists( s_File ) )
	{
		while( i < file_size( s_File, 1 ) && read_file( s_File, i, s_WordString[ iPosStrFile ], 63, iLen ) )
		{
			i++;
			if( s_WordString[ iPosStrFile ][ 0 ] == ';' || iLen == 0 )
			{
				continue;
			}
			iPosStrFile++;
		}
	}
	
	if( file_exists( s_FileReplace ) )
	{
		i = 0;
		while( i < file_size( s_FileReplace, 1 ) && read_file( s_FileReplace, i, s_StringReplace, 255 iLen ) )
		{
			i++;
			if( s_StringReplace[ 0 ] == ';' || iLen == 0 )
			{
				continue;
			}
		}
	}
	else
	{
		write_file( s_FileReplace, "; [ Ignore Words v0.5 ] by BlackSmoke^n; Сюда вписываем строчку, которую будет выводить когда игрок написал игнорируемое слово.^n^n^"Сообщение от ( NAME ) удалено^"" );
	}
	
	if( file_exists( "addons/amxmodx/configs/ignore_words/config.cfg" ) )
	{
		server_cmd( "exec addons/amxmodx/configs/ignore_words/config.cfg" ) ;
		server_exec( ) ;
	}
	
	return PLUGIN_CONTINUE;
}

/**==============================================================================
	Хук чата
================================================================================*/
public Hook__Say( id )
{
	if( get_user_flags( id ) & ADMIN_FLAG && get_pcvar_num( iCvarIgnoreAdmins ) )
	{
		return PLUGIN_CONTINUE;
	}
	
	new s_Msg[ 192 ], s_Name[ 32 ], iWord;
	get_user_name( id, s_Name, charsmax( s_Name ) );
	read_args( s_Msg, charsmax( s_Msg ) );
	remove_quotes( s_StringReplace );
	iWord = is_message_Word( s_Msg );
	if( iWord )
	{
		replace_all( s_Msg, charsmax( s_Msg ), "%", "" );
		replace( s_StringReplace, 255, "NAME", s_Name ) ;
		if( get_pcvar_num( iCvarDeleteOrReplace ) )
			Color_Print_All( "!team%s", s_StringReplace );
			
		replace( s_StringReplace, 255, s_Name, "NAME" ) ;
		return PLUGIN_HANDLED;
	}
	
	return PLUGIN_CONTINUE;
}

/**==============================================================================
	Проверяем сообщение на ругательства.
================================================================================*/
stock is_message_Word( s_Msg[ ] )
{
	for( new i; i < file_size( s_File, 1 ); i++ )
	{
		if( containi( s_Msg, s_WordString[ i ] ) != -1 && s_WordString[ i ][ 0 ])
		{
			return 1;
		}
	}
	
	return 0;
}

/**==============================================================================
	Цветной Чат
================================================================================*/
stock Color_Print_All( const input[], any:... )
{
	static msg[ 191 ];
	new s_Players[ 32 ], iNum;
	get_players( s_Players, iNum );
	vformat( msg, 190, input, 2 );
	format( msg, 190, "!y[!g%s!y] %s", PLUGIN_NAME, msg );
	replace_all( msg, 190, "!g", "^4" );
	replace_all( msg, 190, "!y", "^1" );
	replace_all( msg, 190, "!team", "^3" );
	for( new i; i < iNum; i++ )
	{
		message_begin( MSG_ONE_UNRELIABLE, get_user_msgid("SayText"), _, s_Players[ i ] );
		write_byte( s_Players[ i ] );
		write_string( msg ); 
		message_end( );
	}
}